/** 
 driver routines for adaptive rejection 
 (arms.c) 
*/
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <nicklib.h>
#include "qsubs.h" 
#include "arms.h"
#include "sampgw.h" 

extern int qtmode ; 
extern double qtrho ;

static int debug = NO ;
int armsiter = 0 ;

static double xranb1(double a, double b, double lo, double hi)    ;
static double xranb2(double a, double b, double lo, double hi)    ;
static double betap(double x, void *data) ;

/* ********************************************************************* */

struct beta_parm {
  double a, b ;   
};

struct xparm {
  double xp ;
  double xlam ;
  double xlo ;
  double xhi ;
  double xroll ;
};

struct xparma {
  double xn[2] ;
  double xalpha ;
  double xpp ;
};


struct xparmb {
  double xp1   ;
  double xp2   ;
  double xp3   ;
  double lmean;
  double lsig;
};

struct xparmb2 {
  double xsuma ;
  double xsumb ;
  double n ;
  double ya   ;
  double yb;
  int mode ;
  double lmean;
  double lsig;
  int qmode ;
  double *tt ; 
  double *qv ;
};

struct xparmt {
  double *xp1   ;
  double *xp2   ;
  int n ;
  double lmean   ;
  double lsig ;
};

struct xparmdis {
  double *xdis  ;
  double *priordis   ;
  int n ;
};

struct xparmg {
  double xsum   ;
  double xlsum  ;
  double xn ;
  double pmean ;
  double pvar ;
  int mode ;
};

struct xparmbb {
  double *u   ;
  double *x   ;
  int n ;
  double ya ;
  double yb ;
  double yc ;
  int mode ;
  double lmean;
  double lsig;
};

struct xparmbt {
   double xtheta ;
   double ta ;
   double tb ;
   double a ;
   double b ;
   double c ;
   double qval ;
   double rco[3] ;
} ;

/* ********************************************************************* */

double xsub1(double x, void *xdata)
{
   struct xparm *d;
   double y, t;

   /* cast voided pointer into pointer to struct xparm */
   d = xdata;
   y = (d->xp-1)*log(x) ;
   y -= d->xlam * x ;
   t = 0.0 ;
   if (x< d-> xlo) {  
     t = d->xlo -x ;
   }
   if (x> d-> xhi) {  
     t = x - d->xhi  ;
   }
   if (t==0.0) return y ;
   if (d->xroll==0.0) return y ;
   t /= d->xroll ;
   y -= t*log(2.0) ;
   return y;
}

/** 
 So we have 2 levels  

 Beta (N.n) -> p  -> Beta(p alpha) -> pp

 Two sampling steps.  
 1) Generate p | pp  
 2) Generate pp | p  this is a straightforward beta sampler

*/





double xsuba(double x, void *xdata)
{
   struct xparma *d;
   double y, p, q, pe, qe, alpha;
   double pp, qq ; 

   /* cast voided pointer into pointer to struct xparma */
   d = xdata;
   p = x ;  
   q = 1.0-x ;

   if (p<=0.0) return -1.0e20 ;
   if (q<=0.0) return -1.0e20 ;

   pe = d->xn[0] - 1.0 ;
   qe = d->xn[1] - 1.0 ;
   y = pe*log(p) + qe*log(q) ;
   if (!finite(y)) fatalx("bad xsuba\n") ;
/* this is beta prior on p, q  */
   pp = d->xpp ;
   qq = 1.0-pp ;

   alpha = d->xalpha ;

   y += bprob(pp, alpha*p, alpha*q) ;
   if (!finite(y)) fatalx("bad xsuba\n") ;

/** 
 beta likelihood of P(pp; alpha p, alpha q) 
*/
   return y;
}

double xsubbmu(double x, void *xdata)
{
   struct xparmb *d;
   double xa, xb, y  ;
   double mu ;

   d = xdata;
   xa  = d -> xp1 ;
   xb  = d -> xp2 ;
   mu  = d -> xp3 ;

   y = bprob(x, xa, xb) ;
   y += (-mu*x) ;

   return y ;

}


double xsubb(double x, void *xdata)
{
	   struct xparmg *d;
	   double y, xsum, xlsum ;  
	   double mean, var, xn ;
	   double p, lam ;
	   int mode ;

	   /* cast voided pointer into pointer to struct xparma */
	   d = xdata;
	   xn = d -> xn ;
	   mode = d -> mode ;
	   mean = d->pmean ;
	   var  = d->pvar ;
	   xsum  = d -> xsum ;
	   xlsum = d -> xlsum ;
	   if (mode == 0) mean = x ;
	   if (mode == 1) var = x ;
	   gpars(&p, &lam, mean, var) ;
	   y = (p-1) * xlsum ;
	   y -= lam * xsum ;
	   y += xn*p*log(lam) ;
	   y -= xn*xlgamma(p) ;
// Prior on var:  1/var is exponential mean 1  
// Prior on mean  Gamma 
           y += priorgl(mean, var) ;
	   return y;
}
double  priorgl(double mean, double var)  
{
/** mean var for prior on lambda. hyperprior likelihood */
         extern double priorlmean, priorlmsig ;

         double y ; 
         double p, lam ;

         y = (-2.0*log(var) -1.0/var) ;  
	 gpars(&p, &lam, priorlmean, priorlmsig*priorlmsig) ;
         y += (p-1) * log(mean) ;
         y -= lam * mean ;
	 return y;

}

void gpars(double *p, double *lam, double mean, double var)
{
	  double l ;

          if (mean<0.0) fatalx("(gpars) bad mean\n") ;
          if (var<0.0) fatalx("(gpars) bad var\n") ;

	  l = mean/var ;
	  *p = l*mean ;
	  *lam = l ;
}

double xsubb2(double x, void *xdata)
{
   struct xparmb2 *d;
   double y, xp1, xp2, xp3, t, sig, ts ;  
   double ya, yb, xsuma, xsumb, xn ;
   double *xa, *xb, rco[3], mu, theta ; 
   int mode, n, i ;

   /* cast voided pointer into pointer to struct xparma */
   d = xdata;

   xsuma = d -> xsuma ;
   xsumb = d -> xsumb ;
   xn = (double) ( d -> n ) ;
   mode = d -> mode ;

   ya = d -> ya ;
   yb = d -> yb ;

   sig = d -> lsig ;
 
   y = 0.0 ;  // log likelihood

   if (mode == 1) { 
     ya = x ;
   }
   if (mode == 2) { 
     yb = x ;
   }

   if (mode == 3) { 
    ya *= x ;
    yb *= x ;
    y += 2.0*log(x) - x ;
// jacobian and exponential prior see: mcmcline.tex
   }

   y += (ya-1.0)*xsuma + (yb-1.0) *xsumb ;
   y -= xn*lbeta(ya, yb) ; ;
   t  = log10(ya + yb) ;
   t -= (d -> lmean ) ;
   t /= sig ; 
   y -= log(sig) ; 
   y -= 0.5*t*t ;

   if (d->qmode) {  
     n = d -> n ;
     setqco(rco, ya, yb) ;
     mu = ya/(ya+yb) ;
     for (i=0; i<n; i++)  {  
      theta = d->tt[i] ;
      y += qqlike(d->qv[i], rco, theta, mu, 0) ;
     }
   }

   return y;

}

double xsubt(double x, void *xdata)
{
   struct xparmt *d;
   double y,  y1, *x1, *x2, dev, sig, tau ;  
   int n ;

    ++armsiter ;
   /* cast voided pointer into pointer to struct xparma */
   d = xdata;

   x1 = d -> xp1;
   x2 = d -> xp2 ;
   tau = pow10(x) ;
   n = d -> n ;
   y1 = xtaulike0(tau, x1, x2, n, d -> lmean, d-> lsig) ;
// printf("zz %4d %9.3f %12.3e\n", armsiter, tau, y1) ;
   if (armsiter > 1000) fatalx("too many iters\n") ;
   return (y1);
}

double xsubbb(double x, void *xdata)
{
   struct xparmbb *d;
   double *uu, *xx ;          
   int n ;

   double ya, yb, yc ;
   double xa, xb ;
   int mode ;
   double y ; 
   double t, sig, ts ;

   int i ;

   /* cast voided pointer into pointer to struct xparma */
   d = xdata;

   uu = d -> u ;
   xx = d -> x ;
   n = d->n ;  

   ya = d -> ya ;
   yb = d -> yb ;
   yc = d -> yc ;

   mode = d -> mode ;
   sig  = d -> lsig ;

   if (mode == 1) ya = x ;
   if (mode == 2) yb = x ;
   if (mode == 3) yc = x ;

   yc = MAX(yc, 0.0) ;

   y = 0.0 ;

   if (mode == 4) {  
    ts =  x ;  
    ya *= ts ;
    yb *= ts ;
    yc *= ts ;
    y += 3.0*log(ts) ;
// this term comes from the Jacobian
    y -= ts ;  // prior is exponential mean 1
   }



   for (i=0; i<n; i++) {  
    xa = yc*uu[i] + ya ;
    xb = yc*(1.0-uu[i]) + yb ;
    y  += bprob(xx[i], xa, xb) ;
   }

 
   t  = log10(ya + yb +yc) ;
   t -= (d -> lmean ) ;
   t /= sig ; 
   y -= log(sig) ; 
   y -= 0.5*t*t ;


   return y;

}

double xsubbt(double x, void *xdata)
{
   struct xparmbt *d;
   int n ;

   double a, b, c, ta, tb, xtheta ;
   double xa, xb ;
   double y ; 
   double *rco, mu ;

   /* cast voided pointer into pointer to struct xparmbt */
   d = xdata;

   xtheta = d -> xtheta ;
   ta = d -> ta ;
   tb = d -> tb ;

   a = d -> a ;
   b = d -> b ;
   c = d -> c ;

   y = bprob(x, ta, tb) ;
   xa = c*x +a ;
   xb = c*(1-x) + b ;

   y += bprob(xtheta, xa, xb) ;

   if (qtmode==YES)  { 
    rco = d -> rco ;
    mu = ta/(ta+tb) ;
    y += qqlike(d->qval, rco, x, mu, 0) ;
   }


   return y;

}

double sampit1c(double *xn, double alpha, double pp)  
{

  struct xparma xdata  ;

  int err, ninit = 0, npoint = 101, nsamp = 1, ncent = 4 ;
  int neval,i;
  double xinit[20], xl, xr ;
  double xsamp[10], xcent[10], qcent[10] = {5., 30., 70., 95.} ;
  double convex = 1.;
  int dometrop = 0;
  double xprev, xinc, xa, xb, xx ;


  xdata.xn[0] = xn[0] ;
  xdata.xn[1] = xn[1] ;
  xdata.xalpha = alpha ;
  xdata.xpp = pp ;


     xl = 0.0 ; xr = 1.0 ;

     xa = xn[0] + 100*pp ;
     xb = xn[1] + 100*(1-pp) ;
     xx = xa/(xa+xb) ; /* initial guess */
     xx = MIN(xx, .975) ;
     xx = MAX(xx, .025) ;
     xinc = 0.1  ;
     ninit = 20 ;
     setxinit(xinit, &ninit, xx, xinc, xl, xr,  &xdata, xsuba) ;
     xprev = xx ;

    err = arms(xinit,ninit,&xl,&xr,xsuba, &xdata, &convex,
              npoint,dometrop,&xprev,xsamp,nsamp,
              qcent,xcent,ncent,&neval);


    if(err>0){
      fatalx("***warn (sampit1c) error code = %d\n",err);
    }
    xprev = xsamp[0] ; 
    return xprev ;

}


void sampbmu(double *xval, double xa, double xb, double mu) 
{
/** 
 log p(x, a, b, mu) \propto bprob(x, a, b) - mu * x 
*/
  struct xparmb xdata  ;

  int err, ninit = 3, npoint = 101, nsamp = 1, ncent = 0 ;
  int neval,i;
  double xinit[20], xl, xr, xx ;

  double xsamp[10], xcent[10], qcent[10] = {5., 30., 70., 95.} ;
  double convex = 1.;
  int dometrop = 0;
  double xprev, t  ;

  double maxm, p, minm ;
  int nlarge = 0 ;

  maxm = MAX(1.0, exp(-mu)) ;
  minm = MIN(1.0, exp(-mu)) ;

/** 
 simple  rejection sampler using B(xa, xb) 
 as proposal distribution 
*/
  if ((maxm < 100*minm) || (xa<1) || (xb<1)) {  
    for (;;) {  
      *xval = t = ranbeta(xa, xb) ;
      p = exp(-t*mu)/maxm ;
      if (DRAND() < p) return ;
    }
   }
   if ((xa>=1) && (xb>=1)) ++ nlarge ;
   if (nlarge<5) printf("*** warn... xsubmu large mu %9.3f\n", mu) ;


  if (mu==0.0) {  
    *xval = ranbeta(xa, xb) ;
    return ;
  }

  xdata.xp1 = xa ;
  xdata.xp2 = xb ;
  xdata.xp3 = mu ;

  xl = 0; xr = 1 ;

     xx = *xval ;
     ninit = 20 ;
     setxinit(xinit, &ninit, xx, 1.0, xl, xr,  &xdata, xsubbmu) ;
     xprev = xx ;


    err = arms(xinit,ninit,&xl,&xr,xsubbmu, &xdata, &convex,
              npoint,dometrop,&xprev,xsamp,nsamp,
              qcent,xcent,ncent,&neval);

    if(err>0){
      fatalx("***warn (sampprbmub) error code = %d %9.3f %9.3f\n",
       err, xinit[0], xinit[2]);
    }

    *xval = xsamp[0] ;

}

double bprobt(double p, double taux, double px, 
  double *xd1, double *xd2) 
/* first second der of bprob wrt tau */
{

   double q, yl, qx ;
   double a, b, d1, d2 ;
   q  = 1.0 - p ;
   qx = 1.0 - px ;

   a  = taux*px ;
   b  = taux*qx ;

   yl = (a-1) * log(p) + (b-1) * log (q) ;

   d1 = px*log(p) + qx*log(q) ;   
   d2 = 0.0 ;

   yl -= lbeta(a, b) ;
   d1 -= px * psi(a)  ;
   d1 -= qx * psi(b)  ;
   d2 -= px * px * tau(a)  ;
   d2 -= qx * qx * tau(b)  ;

   d1 += psi(a+b)  ;
   d2 += tau(a+b)  ;

   *xd1 = d1 ; 
   *xd2 = d2 ; 

   return yl ;
}


void sampprg(double *pp, double *plam, double *x, int n, int niter,
 int mlmode)  

{

  struct xparmg xdata  ;

  int err, ninit = 3, npoint = 101, nsamp = 1, ncent = 0 ;
  int neval,i;
  double xinit1[20], xinit2[20], xl, xr, *xinit ;
  int ninit1, ninit2 ;
  double xsamp[10], xcent[10], qcent[10] = {5., 30., 70., 95.};
  double convex = 1.;
  int dometrop = 1;
  int numiter ;

  double p, lam, pmax, lmax ;
  double xsum, xlsum, xn ;
  double xmax, xmin;
  int iter ;
  double mean, var, xinc ;
  double xx, xprev ;
/** 
 the log likelihood here is not always (I think) 
 concave so dometrop ia set 1. 
 Then we call setxinit NOT using the current, p, lambda 
 values, so as to be careful (see the corrigenda in the 
 doc directory).
*/

  numiter = niter ;

  xn = (double) n ;

  xsum = xlsum = 0.0 ;
  for (i=0; i<n; i++) {  
   xsum += x[i] ;
   xlsum += log(x[i]) ;
  }

  xdata.xsum = xsum ;
  xdata.xlsum = xlsum ;
  xdata.xn = xn ;

  p = *pp ;
  lam = *plam  ;

  mleg(xsum/xn, xlsum/xn, &pmax, &lmax) ;
  if (mlmode) {
   *pp = pmax ; 
   *plam = lmax ;
   return ;
  }

  mean = pmax/lmax ;
  var = mean/lmax ;


  if (mean<1.0) numiter = 1 ;
  if (var<0.1)  numiter = 1 ;

  xl = 0.0 ; xr = 1.0e8 ;

  var = MAX(var, xl) ;
  var = MIN(var, xr) ;

  xl = 0.0 ; xr = 1.0e5 ;
  mean = MAX(mean, xl) ;
  mean = MIN(mean, xr) ;

   xdata.mode = 0 ;
   xx = xdata.pmean = mean ;
   xdata.pvar = var ;

   xinc = sqrt(var) ;

   ninit1 = 20 ;
   setxinit(xinit1, &ninit1, xx, xinc, xl, xr,  &xdata, xsubb) ;

  xl = 0.0 ; xr = 1.0e8 ;

   xdata.mode = 1 ;
   xx = xdata.pvar = var ;
   xdata.pmean = mean ;

   xinc = var*0.11 ;

   ninit2 = 20 ;
   setxinit(xinit2, &ninit2, xx, xinc, xl, xr,  &xdata, xsubb) ;

  mean = p/lam ;
  var = mean/lam ;

  for (iter=1; iter <= numiter; ++iter) {  

  xl = 0.0 ; xr = 1.0e5 ;

   xdata.mode = 0 ;
   xx = xdata.pmean = mean ;
   xdata.pvar = var ;

   xinc = sqrt(var) ;

   ninit = ninit1 ;
   xinit = xinit1 ;

   xprev = mean ;

    err = arms(xinit,ninit,&xl,&xr,xsubb, &xdata, &convex,
              npoint,dometrop,&xprev,xsamp,nsamp,
              qcent,xcent,ncent,&neval);

    if(err>0){
      fatalx("***warn (samprg 0) error code = %d %9.3f %9.3f\n",
       err, xinit[0], xinit[2]);
    }

    mean = xsamp[0] ;

  xl = 0.0 ; xr = 1.0e8 ;

   xdata.mode = 1 ;
   xx = xdata.pvar = var ;
   xdata.pmean = mean ;

   ninit = ninit2 ;
   xinit = xinit2 ;

   xprev = var ;
   xsamp[0] = 2.0 ;

    err = arms(xinit,ninit,&xl,&xr,xsubb, &xdata, &convex,
              npoint,dometrop,&xprev,xsamp,nsamp,
              qcent,xcent,ncent,&neval);

    if(err>0){
      fatalx("***warn (samprg 0) error code = %d %9.3f %9.3f\n",
       err, xinit[0], xinit[2]);
    }

 
    var = xsamp[0] ;

   }

   gpars(&p, &lam, mean, var) ;

/* 
 1/variance has prior exponential mean 1
 mean  exponential mean priormean = 6
 having sampled mean, variance convert back to p, lambda coords
*/

   *pp = p ;
   *plam = lam ;

}

void sampprb(double *pa, double *pb, double *qq, double *x, 
 int n, int niter, double lmean, double lsig, int qmode)  

{
/** 
 sample prior on beta 
 insist a, b >= 1
 Normal prior on log(a+b)
*/

  struct xparmb2 xdata, *d  ;

  int err, ninit = 3, npoint = 101, nsamp = 1, ncent = 0 ;
  int neval,i;
  double xinit[20], xl, xr, xl3, xr3, oldxl3, oldxr3 ;
  double xsamp[10], xcent[10], qcent[10] = {5., 30., 70., 95.};
  double convex = 1.;
  int dometrop = 0;
  double xprev, t  ;
  static int nbad = 0 ;

  double xa, xb ;
  double xsuma, xsumb, xn ;
  double xmin, xmax, ts ;
  double oldxa, oldxb ;
  int iter ;

  xn = (double) n ;

  xl = 1 ; xr = 1.0e8 ;

  xsuma = xsumb = 0.0 ;
  vmaxmin(x, n, &xmax, &xmin) ;


  if (xmax>= 1.0) fatalx("bad xmax\n") ;
  if (xmin<= 0.0) fatalx("bad xmin\n") ;

  for (i=0; i<n; i++) {  
   xsuma += log(x[i]) ;
   xsumb += log(1.0-x[i]) ;
  }

  xa =   *pa ;
  xb =   *pb ;

  xa = MIN(xa, xr) ;
  xa = MAX(xa, xl) ;
  xb = MIN(xb, xr) ;
  xb = MAX(xb, xl) ;
  

  xdata.lmean = lmean ; 
  xdata.lsig = lsig ; 
  xdata.xsuma = xsuma ;
  xdata.xsumb = xsumb ;
  xdata.n = n ;
  xdata.qmode = qmode ;

  d = &xdata ;
  d -> tt = d -> qv = NULL ;
  if (qmode) {  
   d -> tt = x ;
   d -> qv = qq ;
  }
  
  for (iter=1; iter <= niter; ++iter) {  

   xl = 1 ; xr = 1.0e6 ;
   xdata.ya = xa ;  xdata.yb = xb ;
   xdata.mode = 2 ;

     ninit = 20 ;
     setxinit(xinit, &ninit, xb, 1.0, xl, xr,  &xdata, xsubb2) ;
     xprev = xb ;


    err = arms(xinit,ninit,&xl,&xr,xsubb2, &xdata, &convex,
              npoint,dometrop,&xprev,xsamp,nsamp,
              qcent,xcent,ncent,&neval);

    if(err>0){
      fatalx("***warn (sampprb 2) error code = %d %9.3f %9.3f% 9.3f %9.3f\n",
       err, xinit[0], xinit[2], xl, xr);
    }

    xb = xsamp[0] ;

   xl = 1 ; xr = 1.0e6 ;
   xdata.ya = xa ;  xdata.yb = xb ;
   xdata.mode = 1 ;

     ninit = 20 ;
     setxinit(xinit, &ninit, xa, 1.0, xl, xr, &xdata, xsubb2) ;
     xprev = xa ;

    err = arms(xinit,ninit,&xl,&xr,xsubb2, &xdata, &convex,
              npoint,dometrop,&xprev,xsamp,nsamp,
              qcent,xcent,ncent,&neval);


    if(err>0){
      fatalx("***warn (sampprb 1) error code = %d %9.3f %9.3f% 9.3f %9.3f\n",
       err, xinit[0], xinit[2], xl, xr);
    }

    xa = xsamp[0] ;

   xl = 1 ; xr = 1.0e6 ;

    t = ranexp() ;   

    oldxa = xa ;
    oldxb = xb ;
    xa = xa/t ;
    xb = xb/t ;
    xdata.ya  = xa ;
    xdata.yb  = xb ;
    xdata.mode = 3 ;

    xl3 = MAX(xl/xa, xl/xb) ;
    xr3 = MIN(xr/xa, xr/xb) ;
    
    if ((t<=xl3) || (t>=xr3)) {  

      printf("zzskipx %9.3f %9.3f %9.3f\n", t, xl3, xr3) ;

      xa *= t ;
      xb *= t ;

      continue ;
    }
    
    ninit = 20 ;
    setxinit(xinit, &ninit, t, t / 100 ,  xl3, xr3, &xdata, xsubb2) ;
    xprev = t ;

    err = arms(xinit,ninit,&xl3,&xr3,xsubb2, &xdata, &convex,
              npoint,dometrop,&xprev,xsamp,nsamp,
              qcent,xcent,ncent,&neval);

    if(err>0){

      ++nbad ;
      if (nbad <= 10) { 
       printf ("bad sampprb (mode 3)\n") ; 
       printf("***warn (sampprb) error code = %d %9.3f %9.3f\n",
        err, xinit[0], xinit[2]);
      }
// probably not a true gibbs sample but case does not occur with real data
      dometrop = YES ;

       err = arms(xinit,ninit,&xl3,&xr3,xsubb2, &xdata, &convex,
              npoint,dometrop,&xprev,xsamp,nsamp,
              qcent,xcent,ncent,&neval);
      if (nbad <= 10) 
       printf("err (metropolis): %d  %12.6f %12.6f\n", err, xprev, xsamp[0]) ;

    }


    t = xsamp[0] ;
    xa = xdata.ya * t ;
    xb = xdata.yb * t ;
//  printf("zzzq %9.3f    %9.3f %9.3f\n", t, oldxa, xa) ;
   }

   *pa = xa ;
   *pb = xb ;

}

double sampprt(double *ptau, double *x1, double *x2, int n, 
	  double lmean, double lsig)   

{

	  struct xparmt xdata  ;

	  int err, ninit = 0, npoint = 101, nsamp = 1, ncent = 0 ;
	  int neval,i;
	  double xinit[20], xl, xr ;
	  double xsamp[50], xcent[10], qcent[10] = {5., 30., 70., 95.};
	  double convex = 1.;
	  int dometrop = 1;  // change by NJP
	  static double xprev = 100 ;

	  double ltau, ltry ; 
	  double tau, xval, xv2, xv1 ;
	  int iter ;

	  ninit = 0; 
          armsiter = 0 ;

	  tau = *ptau ;
	  ltau = log10(tau) ;


	  xdata.xp1 = x1 ;
	  xdata.xp2 = x2 ;
	  xdata.lmean = lmean ;
  xdata.lsig = lsig ;
  xdata.n = n ;

  xl = 1.0 ; // min value of tau is 10
  xr = 15.0 ; 
  ninit = 20 ;
  setxinit(xinit, &ninit, ltau, 0.1, xl, xr, &xdata, xsubt) ;

  xval = xsubt(ltau, &xdata) ; 
  if (fabs(xval) > 1.0e10) { 
   debug = YES ;
   xval = xsubt(ltau, &xdata) ; 
   fatalx("bad prt\n") ;
  }

  xprev = ltau ;

    err = arms(xinit,ninit,&xl,&xr,xsubt, &xdata, &convex,
              npoint,dometrop,&xprev,xsamp,nsamp,
              qcent,xcent,ncent,&neval);


    if(err>0){
      printf("prt bug %9.3f %d\n", *ptau, ninit) ;
      printmat(xinit, 1, ninit) ;
      printf("ltau: %9.3f\n", ltau) ;
      fatalx("***error (sampprt) error code = %d\n",err);
    }
    xprev = ltau = xsamp[0] ;
    *ptau = pow10(ltau) ;
    xval = xsubt(ltau, &xdata) ;
    return xval ;

}

double xtaulike0(double tau, double *x1, double *x2, int n, 
 double lmean, double lsig) 
{
   double y0, y1, ltau ;
   double t, dev ;
   int i ;


   y0 = xtaulike(tau, x1, x2, n) ;  
   if (!finite(y0)) fatalx("internal bug\n") ;
   if (lsig <= 0.0) return y0 ;

    ltau = log10(tau) ; 
    dev = ltau - lmean ;  
    t = dev/lsig ;  
    y1 = -0.5*t*t ;

   return (y0+y1) ;

}

double xtaulike(double tau, double *x1, double *x2, int n) 
{
/** 
 prior info comes from x1 
 actual probs comes from x2 
*/
      int i ; 
      double ylike, ya, yb, yl, y, p ;
      double tauq = 100.0 ;

      ylike = 0.0 ;
      for (i=0; i<n; i++)  {  

        p =  x1[i] ;        
        ya = tau * p ;
        yb = tau * (1.0-p) ; 
        y = bprob(x2[i] , ya, yb) ;
        if (!finite(y)) fatalx("zzbprob %9.3f %9.3f %9.3f %9.3f\n", 
          x2[i], ya, yb, y) ;
        
        ylike += y ;
      }
      return ylike ;
}

void sampprbt(double *theta, double xtheta, 
 double ta, double tb, double c, double a, double b, double qval) 

{
  struct xparmbt xdata, *d  ;

  int err, ninit = 3, npoint = 101, nsamp = 1, ncent = 0 ;
  int neval,i;
  double xinit[10], xl, xr ;
  double xsamp[10], xcent[10], qcent[10] = {5., 30., 70., 95.};
  double convex = 1.;
  int dometrop = 0;
  double xprev, th, *rr ;

  static int ncall=0, nbad = 0 ;
  
  ++ncall ;

  xdata.xtheta = xtheta ; 
  xdata.ta = ta ; 
  xdata.tb = tb ; 
  xdata.a = a ; 
  xdata.b = b ; 
  xdata.c = c ; 

  if (qtmode) {  
   d = &xdata ;
   d -> qval = qval ;
   rr = d -> rco ;
   setqco(rr, ta, tb) ;
  }

  th = *theta ;

  xinit[0] = th/2 ;
  xprev = xinit[1] = th ; 
  xinit[2] = th + 0.5 * (1.0-th) ;

  xl = 0.0 ; xr = 1.0 ;

    err = arms(xinit,ninit,&xl,&xr,xsubbt, &xdata, &convex,
              npoint,dometrop,&xprev,xsamp,nsamp,
              qcent,xcent,ncent,&neval);

    if(err>0){

      ++nbad ;
      if (nbad <= 10) { 
       printf ("bad sampprbt\n") ; 
       printf("ta,tb: %12.6f %12.6f\n",ta,tb) ;
       printf("a,b,c: %12.6f %12.6f %12.6f\n",a,b,c) ;
       printf("theta, xtheta: %12.6f %12.6f\n",th, xtheta) ;
       printf("***warn (sampprbt) error code = %d %9.3f %9.3f\n",
        err, xinit[0], xinit[2]);
      }
// probably not a true gibbs sample but case does not occur with real data
      dometrop = YES ;
      err = arms(xinit,ninit,&xl,&xr,xsubbt, &xdata, &convex,
              npoint,dometrop,&xprev,xsamp,nsamp,
              qcent,xcent,ncent,&neval);

      if (nbad <= 10) 
       printf("err (metropolis): %d  %12.6f %12.6f\n", err, xprev, xsamp[0]) ;

    }

    *theta  = xsamp[0] ;


}

void sampprbb(double *pc, double *pa, double *pb, 
 double *u, double *x, int n, int niter, double lmean, double lsig)  

{

  struct xparmbb xdata  ;

  int err, ninit = 3, npoint = 101, nsamp = 1, ncent = 0 ;
  int neval,i;
  double xinit[20], xl, xr, xl4, xr4, xl3  ;
  double xsamp[10], xcent[10], qcent[10] = {5., 30., 70., 95.};
  double convex = 1.;
  int dometrop = 1;
  double xprev  ;
  int nbad = 0 ;

  double ya, yb, yc, t ;
  double xmin, xmax, ts ;
  int iter ;

  xl = 1.0 ; xr = 1.0e6 ;
  xl3 = 0.0 ;

  vmaxmin(x, n, &xmax, &xmin) ;
  if (xmax> 1.0) fatalx("bad xmax\n") ;
  if (xmin< 0.0) fatalx("bad xmin\n") ;

  vmaxmin(u, n, &xmax, &xmin) ;
  if (xmax> 1.0) fatalx("bad xmax\n") ;
  if (xmin< 0.0) fatalx("bad xmin\n") ;


  ya =   *pa ;
  yb =   *pb ;
  yc =   *pc ;

//printf("zzprbbinit %9.3f %9.3f %9.3f \n", ya, yb, yc) ;

  xdata.lmean = lmean ;
  xdata.lsig = lsig ;
  
  for (iter=1; iter <= niter; ++iter) {  

//printf("zziter %d %9.3f %9.3f %9.3f\n", iter, ya, yb, yc) ;
//fflush(stdout) ;

   xl = 1.0 ; xr = 1.0e6 ;

   xdata.u =  u ;  
   xdata.x =  x ;
   xdata.n =  n ;

   xdata.ya  = ya ;
   xdata.yb  = yb ;
   xdata.yc  = yc ;

   xdata.mode = 2 ;

     ninit = 20 ;
     setxinit(xinit, &ninit, yb, 1.0, xl, xr, &xdata, xsubbb) ;
     xprev = yb ;


    err = arms(xinit,ninit,&xl,&xr,xsubbb, &xdata, &convex,
              npoint,dometrop,&xprev,xsamp,nsamp,
              qcent,xcent,ncent,&neval);

    if(err>0){
      fatalx("***warn (sampprbb 2) error code = %d %9.3f %9.3f\n",
       err, xinit[0], xinit[2]);
    }

    yb = xsamp[0] ;

   xdata.u =  u ;  
   xdata.x =  x ;
   xdata.n =  n ;

   xdata.ya  = ya ;
   xdata.yb  = yb ;
   xdata.yc  = yc ;
   xdata.mode = 1 ;

   xl = 1.0 ; xr = 1.0e6 ;


     ninit = 20 ;
     setxinit(xinit, &ninit, ya, 1.0, xl, xr, &xdata, xsubbb) ;
     xprev = ya ;

    err = arms(xinit,ninit,&xl,&xr,xsubbb, &xdata, &convex,
              npoint,dometrop,&xprev,xsamp,nsamp,
              qcent,xcent,ncent,&neval);


    if(err>0){
      fatalx("***warn (sampprbb 1) error code = %d %9.3f %9.3f\n",
       err, xinit[0], xinit[2]);
    }

    ya = xsamp[0] ;

   if (yc < 0) continue ;
   xdata.u =  u ;  
   xdata.x =  x ;
   xdata.n =  n ;

   xdata.ya  = ya ;
   xdata.yb  = yb ;
   xdata.yc  = yc ;
   xdata.mode = 3 ;

   xl3 = 0.0 ; xr = 1.0e6 ;

    ninit = 20 ;
    setxinit(xinit, &ninit, yc, 1.0, xl3, xr, &xdata, xsubbb) ;
    xprev = yc ;

    err = arms(xinit,ninit,&xl3,&xr,xsubbb, &xdata, &convex,
              npoint,dometrop,&xprev,xsamp,nsamp,
              qcent,xcent,ncent,&neval);

    if(err>0){

      ++nbad ;
      if (nbad <= 10) { 
       printf ("bad sampprbb mode 3\n") ; 
       printf("***warn (sampprbb) error code = %d %9.3f %9.3f\n",
        err, xinit[0], xinit[2]);
      }
// probably not a true gibbs sample but case does not occur with real data
      dometrop = YES ;
      err = arms(xinit,ninit,&xl3,&xr,xsubbb, &xdata, &convex,
              npoint,dometrop,&xprev,xsamp,nsamp,
              qcent,xcent,ncent,&neval);


    }

    yc = xsamp[0] ;

/* try and improve mixing */
/* see mcmcline.tex for theory */

   t = ranexp() ;  
   if (t==0.0) continue ;

   ya /= t ;
   yb /= t ;
   yc /= t ;

   xdata.ya  = ya ;
   xdata.yb  = yb ;
   xdata.yc  = yc ;
   xdata.mode = 4 ;


    xl = 1.0 ; xr = 1.0e6 ;

    xl4 = MAX(xl/ya, xl/yb) ;
    xr4 = MIN(xr/ya, xr/yb) ;
    
    if ((t<=xl4) || (t>=xr4)) {  

      printf("zzskip %9.3f %9.3f %9.3f\n", t, xl4, xr4) ;

      ya *= t ;
      yb *= t ;
      yc *= t ;

      continue ;
    }
    
    ninit = 20 ;
    setxinit(xinit, &ninit, t, 0.1*t, xl4, xr4, &xdata, xsubbb) ;
    xprev = t ;

    err = arms(xinit,ninit,&xl4,&xr4,xsubbb, &xdata, &convex,
              npoint,dometrop,&xprev,xsamp,nsamp,
              qcent,xcent,ncent,&neval);


    if(err>0){

      ++nbad ;
      if (nbad <= 10) { 
       printf ("bad sampprbb (mode 4)\n") ; 
       printf("***warn (sampprbb) error code = %d %9.3f %9.3f\n",
        err, xinit[0], xinit[2]);
      }
// probably not a true gibbs sample but case does not occur with real data
      dometrop = YES ;
      err = arms(xinit,ninit,&xl4,&xr4,xsubbb, &xdata, &convex,
              npoint,dometrop,&xprev,xsamp,nsamp,
              qcent,xcent,ncent,&neval);

      if (nbad <= 10) 
       printf("err (metropolis): %d  %12.6f %12.6f\n", err, xprev, xsamp[0]) ;

    }

    dometrop = NO ;

    ts = xsamp[0] ; 

    ya *= ts ;
    yb *= ts ;
    yc *= ts ;
  
    /**
    printf("zzexp %d %9.3f %9.3f %9.3f %9.3f\n", iter, ts, ya, yb, yc) ;
    */


   }

   *pa = ya ;
   *pb = yb ;
   *pc = yc ;


}

void setxinit(double *xinit, int *maxn, 
 double x, double inc, double lo, double hi, void *xdata,
   double (*func)(double x, void *xdata) )

/** 
 search around x to find max 
 searches on  a grid with inc  
 inc should be small compared with (hi-lo)
*/

{
#define MAXN 100 
    double xx[MAXN] ;
    double  y ;
    int mxn, n, iter, niter  ;
    double vmax, v, xmin, xmax  ;
  
    mxn = *maxn ;
    mxn = MIN(mxn, MAXN) ;

    inc = MIN(inc, 0.2*(hi-lo)) ;
    if (inc <=0.0) fatalx("(setxinit) bad inc %g %g\n", lo, hi) ;

    if (x <= lo) x = lo+ 1.1*inc ;
    if (x >= hi) x = hi- 1.1*inc ;

    vmax = func(x, xdata) ;
    xx[0]  = x ;

    xmin = xmax = x ;
    y = x ; n=1 ;
    niter = mxn/2 ;
    for (iter = 1; iter <= niter; ++iter) {  
     y += inc ;
     if (y>=hi) break ;
     if (n>=mxn) break ;
     v = func(y, xdata) ;
     xx[n] = y ;
     xmax = y ;
     ++n ;
     if (v<vmax) break ;
     vmax = v ;
    }
    y = x ;
    for (iter = 1; iter <= niter; ++iter) {  
     y -= inc ;
     if (y<=lo) break ;
     if (n>=mxn) break ;
     v = func(y, xdata) ;
     xx[n] = y ;
     xmin = y ;
     ++n ;
     if (v<vmax) break ;
     vmax = v ;
    }

   if (n<3) {  
    y = xmin-0.5*(xmin-lo) ;
    xx[n] = y ;
    ++n ;
   }

   if (n<3) {  
    y = xmax+0.5*(hi-xmax) ;
    xx[n] = y ;
    ++n ;
   }

   sortit(xx, NULL, n) ;
   copyarr(xx, xinit, n) ;

   *maxn = n ;
   return ;

}

double xranbeta(double a, double b, double lo, double hi) 
// sample beta distribution constrained to lie in interval 
// method: 1) try trivial sampling of beta (10  tries) 
// 2) if beta likelihood log-concave use arms 
// 3) else take envelope function from log-concave beta 
// and use (non-adaptive) rejection sampling.  
// There should be an easier simpler method  

{
  int iter ; 
  double aa, m, x1, x2, x3, xx, z, y ;

  if (a<b) return 1.0-xranbeta(b, a, 1.0-hi, 1.0-lo) ;
  z = xranb0(a, b, lo, hi, 10) ; // see if easy case */
  if (z>=0.0) return z ;
  if (b>=1.0)   return xranb2(a, b, lo, hi) ;  
  if (hi>=1.0)  return xranb1(a, b, lo, hi) ;
  if (lo<=0.0) fatalx("(bad xranbeta) %9.3f %9.3f\n", a, lo) ;
  if (hi>=1.0) fatalx("(bad xranbeta) %9.3f %9.3f\n", b, hi) ;

  aa = a/b ;
  m = a/(a+b) ;
  if (m<=lo)  m = lo ;     
  if (m>=hi)  m = hi ;
  x1 = (aa-a)*log(m)+(1-b)*log(1.0-m) ;
  x2 = (aa-a)*log(lo)+(1-b)*log(1.0-lo) ;
  x3 = (aa-a)*log(hi)+(1-b)*log(1.0-hi) ;

  xx = MAX(x1, x2) ;
  x1 = xx = MAX(xx, x3) ;

// now use rejection sampling with ARMS used to sample envelope 
  for (iter=1; iter<=100000; ++iter)  {  
   z = xranb2(aa, 1.0, lo, hi) ;
   xx = (aa-a)*log(z)+(1-b)*log(1.0-z) ;
   xx -= x1 ;
   y = exp(xx) ; 
   if (DRAND() < y) return z ;
  }
  printf("(xranbeta) looping: %9.3f %9.3f %9.3f %9.3f\n", a, b, lo, hi) ;
  return xranb1(a, b, lo, hi) ;
}

double xranb0(double a, double b, double lo, double hi, int numiter) 
{  
  int iter ; 
  double z ;
  for (iter=1; iter<=numiter; ++iter)  {  
   z = ranbeta(a, b) ;
   if (z<lo) continue ;
   if (z>hi) continue ;
   return z ;
  }
  return -1.0 ;  
}

double xranb1(double a, double b, double lo, double hi) 
{  
  int iter ; 
  double z ;
  z = xranb0(a, b, lo, hi, 1000) ;  
  if (z>=0) return z ;
  z = ranbeta(a,b) ;
  z = MAX(z, lo) ;
  z = MIN(z, hi) ;
  return z ;
}

double xranb2(double a, double b, double lo, double hi)
{
  int err, ninit = 4, npoint = 100, nsamp = 1, ncent = 4 ;
  int neval,i;
  double xinit[10],  xl = 0.0, xr = 1.0;
  double xsamp[100], xcent[10], qcent[10] = {5., 30., 70., 95.};
  unsigned seed = 44;
  double convex = 1.;
  int dometrop = 0;
  double xprev = 0.5;
  double xinc = 0.5, xtop;

  /* set up structures for each density function */

  struct beta_parm bdata;

 if (a<b) return 1.0-xranb2(b, a, 1.0-hi, 1.0-lo) ;

 bdata.a = a ;
 bdata.b = b ;

  xl = lo   ; xr = hi   ;
  xinc = (xr-xl)*0.1 ;

  xtop = xr-xinc*0.01 ;  
  xinit[0] = xtop-2.0*xinc ;
  xinit[1] = xtop-1.0*xinc ;
  xinit[2] = xtop ;
  ninit = 3 ;

    err = arms(xinit,ninit,&xl,&xr,betap,&bdata,&convex,
              npoint,dometrop,&xprev,xsamp,nsamp,qcent,xcent,ncent,&neval);
    if(err>0){
      fprintf(stderr,"error code = %d\n",err);
      exit(1);
    }
  return xsamp[0];
}

double betap(double x, void *data)
/* beta log density */
{
   struct beta_parm *d;
   double y, a, b;

   /* cast voided pointer into pointer to struct norm_parm */
   d = data;
   a = d -> a ;
   b = d -> b ;

   y = (a-1)*log(x) + (b-1)*log(1-x) ;
   return y;
}


double xsubdis(double x, void *xdata)
{
	   struct xparmdis *d;

           double *xdis, *priordis ;
           double p, lam, val, y, ylike ;

           int n ; 
           int i ;

	   /* cast voided pointer into pointer to struct xparma */
	   d = xdata;
           xdis = d -> xdis ;
           priordis = d -> priordis ;
	   n = d -> n ;
     

            ylike =  0.0 ;
 

           for (i=0; i<n; i++) {  
            p = x*priordis[i] ; 
            lam = x ;
            val = xdis[i] ;
            val = MAX(val, 1.0e-8) ;
	    y = (p-1) * log(val)  ;
	    y -= lam * val ;
	    y += p*log(lam) ;
	    y -= xlgamma(p) ;
            ylike += y ;
           }

	   return ylike ;
}

void samppdis(double *xflat, double *xdis, double *priordis, int n, double lo, double hi)  
{

  struct xparmdis xdata  ;

  int err, ninit = 0, npoint = 101, nsamp = 1, ncent = 4 ;
  int neval,i;
  double xinit[20], xl, xr ;
  double xsamp[10], xcent[10], qcent[10] = {5., 30., 70., 95.} ;
  double convex = 1.;
  int dometrop = 0;
  double xprev, xinc, xa, xb, xx ;


  xdata.n = n ;
  xdata.priordis = priordis ;
  xdata.xdis = xdis ;


     xl = lo ; xr = hi ;

     xx = *xflat ;
     if ((xx<lo) || (xx>hi))  {
      xx = lo + 0.2 * (hi-lo)  ;  /* initial guess */ 
     }
     xinc = 0.1*(xx-lo)  ;
     ninit = 20 ;
     setxinit(xinit, &ninit, xx, xinc, xl, xr,  &xdata, xsubdis) ;
     xprev = xx ;

    err = arms(xinit,ninit,&xl,&xr,xsubdis, &xdata, &convex,
              npoint,dometrop,&xprev,xsamp,nsamp,
              qcent,xcent,ncent,&neval);


    if(err>0){
      fatalx("***warn (sampit1c) error code = %d\n",err);
    }
    xprev = xsamp[0] ; 
    *xflat = xprev ; 

}

